using System; 
using System.Text;
using System.Collections;
using System.Drawing.Design;
using System.ComponentModel;
using System.ComponentModel.Design;
using System.Diagnostics;
using Borland.Studio.ToolsAPI;

namespace ArtCSB
{
	//-------------------------------------------------------------------------------
	public class ArtCSB_OTAException : Exception
	{
		public ArtCSB_OTAException (string message) : base(message){}
	}
	//-------------------------------------------------------------------------------
	/*
	public class OTASnap{
		//---------------------------------------------------------------------------
		private IOTASourceEditor _CurrentSourceEditor = null;
		private IOTAEditView _CurrentEditView = null;
		private string _CurrentEditorFileName = "";
		private string _CurrentSourceEditorText = "";
		private IOTAModule _CurrentModule = null;
		private IOTAEditor _CurrentEditor = null;
		private IOTAElideActions _CurrentElideActions = null;
		//---------------------------------------------------------------------------
		public IOTASourceEditor CurrentSourceEditor{
			get{ return _CurrentSourceEditor; }
		}
		//---------------------------------------------------------------------------
		public IOTAEditView CurrentEditView{
			get{ return _CurrentEditView;}
		}
		//---------------------------------------------------------------------------
		public string CurrentEditorFileName{
			get{ return _CurrentEditorFileName;}
		}
		//---------------------------------------------------------------------------
		public IOTAModule CurrentModule{
			get{ return _CurrentModule;}
		}
		//---------------------------------------------------------------------------
		public IOTAEditor CurrentEditor{
			get{ return _CurrentEditor;}
		}
		//---------------------------------------------------------------------------
		public string CurrentSourceEditorText{
			get{ return _CurrentSourceEditorText; }
		}
		//---------------------------------------------------------------------------
		public IOTAElideActions CurrentElideActions{
			get{ return _CurrentElideActions; }
		}
		//---------------------------------------------------------------------------
		public void Init(){
			_CurrentSourceEditor = OTAUtils.CurrentSourceEditor;
			_CurrentEditView = OTAUtils.CurrentEditView;
			_CurrentEditorFileName = OTAUtils.CurrentEditorFileName;
			_CurrentModule = OTAUtils.CurrentModule;
			_CurrentEditor = OTAUtils.CurrentEditor;
			_CurrentSourceEditorText = OTAUtils.CurrentSourceEditorText;
			_CurrentElideActions = OTAUtils.CurrentElideActions;
		}
		//---------------------------------------------------------------------------
	}
	*/
	//-------------------------------------------------------------------------------
	public class OTAUtils
	{
		public static readonly int keyShift = 0x2000;
		public static readonly int keyControl = 0x4000;
		public static readonly int keyAlt = 0x8000;

		public static int KeyAlt{
			get{
				return keyAlt;
			}
		}

		private static ArrayList MenuItemsToUpdate = new ArrayList(10);
		private static ArrayList MenuItemShortcuts = new ArrayList(10);
		private static FileNotificationHandler fileNotification = new FileNotificationHandler(UpdateMenuItemShortcuts);

		private static void StartFileNotification()
		{
			IOTAService ideService = OTAService;
			fileNotification = new FileNotificationHandler(UpdateMenuItemShortcuts);
			ideService.FileNotification += fileNotification;
		}
		//---------------------------------------------------------------------------
		public static IOTAService OTAService
		{
			get
			{
				IOTAService service = (IOTAService)
					BorlandIDE.GetService(typeof(IOTAService));
				if (service == null)
				  throw new ArtCSB_OTAException("OTAUtils can\'t obtain OTA service");
				return service;
			}
		}
		//---------------------------------------------------------------------------
		private static void UpdateMenuItemShortcuts(object sender, FileNotificationEventArgs args)
		{
			if ((args.NotifyCode == OTAFileNotification.ofnPackageInstalled) ||
				(args.NotifyCode == OTAFileNotification.ofnPackageUninstalled)  ||
				(args.NotifyCode == OTAFileNotification.ofnFileOpened))
			{
				Debug.Assert(MenuItemsToUpdate.Count == MenuItemShortcuts.Count);
				for (int i = 0; i < MenuItemsToUpdate.Count; i++)
					((IOTAMenuItem)MenuItemsToUpdate[i]).Shortcut =	(int)MenuItemShortcuts[i];
			}
		}
		//---------------------------------------------------------------------------
		public static void StartUpdatingMenuShortcut(IOTAMenuItem item, int shortcut)
		{
			Debug.Assert(MenuItemsToUpdate.Count == MenuItemShortcuts.Count);
			if (item == null)
				return;
			MenuItemsToUpdate.Add(item);
			MenuItemShortcuts.Add(shortcut);
			if (MenuItemsToUpdate.Count == 1)
				StartFileNotification();
		}
		//---------------------------------------------------------------------------
		public static IOTAAboutBoxService AboutBoxService
		{
			get
			{
				IOTAAboutBoxService aboutService = (IOTAAboutBoxService)
					BorlandIDE.GetService(typeof(IOTAAboutBoxService));
				if (aboutService == null)
					throw new ArtCSB_OTAException("OTAUtils can\'t obtain about service");
				return aboutService;
			}
		}
		//---------------------------------------------------------------------------
		public static IOTAMainMenuService MainMenuService
		{
			get
			{
				IOTAMainMenuService menuService = (IOTAMainMenuService)
					BorlandIDE.GetService(typeof(IOTAMainMenuService));
				if (menuService == null)
				  throw new ArtCSB_OTAException("OTAUtils can\'t obtain main menu service");
				return menuService;
			}
		}
		//---------------------------------------------------------------------------
		//---------------------------------------------------------------------------
		public static IOTAEditView CurrentEditView{
			get{
				IOTASourceEditor sourceEditor = OTAUtils.CurrentSourceEditor;
				if ( (sourceEditor != null) && (sourceEditor.EditViewCount > 0) ){
					return sourceEditor.GetEditView(0);
				} else {
					return null;
				}
			}
		}
		//---------------------------------------------------------------------------
		public static IOTAElideActions CurrentElideActions{
			get{
				return (OTAUtils.CurrentEditView as IOTAElideActions);
			}
		}
		//---------------------------------------------------------------------------
		public static string CurrentSourceEditorText{
			get{
				IOTASourceEditor sourceEditor = OTAUtils.CurrentSourceEditor;
				if (sourceEditor == null)
					return "";

				IOTAFileReader fileReader = sourceEditor.CreateReader();

				if (fileReader == null)
					throw new Exception("No file reader");

				System.Text.ASCIIEncoding encoding = new System.Text.ASCIIEncoding();
				StringBuilder stringBuilder = new StringBuilder();
				Byte[] source = fileReader.Read(1024, 0);

				while (source.Length != 0) {
					stringBuilder.Append(encoding.GetString(source));

					source = fileReader.Read(1024, 0);
				}

				return stringBuilder.ToString();
			}
		}
		//---------------------------------------------------------------------------
		private static IOTASourceEditor GetSourceEditor(IOTAModule module) {
			if (module == null) {
				return null;
			}

			if ( CurrentEditor != null &&  GetSourceEditorFromEditor(module, CurrentEditor) != null) {
				return (IOTASourceEditor) GetSourceEditorFromEditor(module, CurrentEditor);
			} else { // Find a source Editor
				IOTAEditor editor;

				for (int i = 0; i < module.ModuleFileCount; i++) {
					editor = module.ModuleFileEditors(i);

					if (editor is IOTASourceEditor) {
						return (editor as IOTASourceEditor);
					}
				}
			}

			return null;
		}

		//---------------------------------------------------------------------------
		public static IOTASourceEditor GetSourceEditorFromEditor(IOTAModule aModule, IOTAEditor aEditor) {
			if (aModule == null || aEditor == null) {
				return null;
			}

			IOTAEditor _Editor;

			for (int i = 0; i < aModule.ModuleFileCount; i++) {
				_Editor = aModule.ModuleFileEditors(i);

				if (_Editor.FileName == aEditor.FileName) {
					if (_Editor is IOTASourceEditor) {
						return (_Editor as IOTASourceEditor);
					} else {
						return null;
					}
				}
			}

			return null;
		}
		//---------------------------------------------------------------------------
		public static void GoToPosition(IOTASourceEditor sourceEditor, int Line, short Col){
			if (sourceEditor != null){
				IOTAEditView editView = CurrentEditView;

				OTAEditPos pos = new OTAEditPos();
				pos.Col = Col;
				pos.Line = Line;

				editView.CursorPos = pos;
			}
		}
		//---------------------------------------------------------------------------
		internal static void CheckNull(object obj, string ServiceName){
			if (obj == null){
				throw new ArtCSB_OTAException("OTAUtils can\'t obtain "+ServiceName);
			}
		}
		//---------------------------------------------------------------------------
		public static IOTASourceEditor CurrentSourceEditor{
			get{
				IOTAModule module = CurrentModule;
				return GetSourceEditor(module);
			}
		}
		//---------------------------------------------------------------------------
		public static IOTAModuleServices ModuleServices{
			get{
				IOTAModuleServices moduleServices =(IOTAModuleServices)BorlandIDE.
					GetService(typeof(IOTAModuleServices));
				CheckNull(moduleServices,
					"ModuleServices");
				return moduleServices;
			}
		}
		//---------------------------------------------------------------------------
		public static IOTADotNetModule CurrentDotNetModule{
			get{
				IOTADotNetModule _dotNetModule =
					(IOTADotNetModule)OTAUtils.ModuleServices.CurrentModule.
					GetService(typeof(IOTADotNetModule));
				CheckNull(_dotNetModule,
					"DotNetModule");
				return _dotNetModule;
			}
		}
		//---------------------------------------------------------------------------
		public static ISelectionService SelectionService{
			get{
				ISelectionService _selectionService =
					(ISelectionService)OTAUtils.CurrentDotNetModule.DesignerHost.
						RootComponent.Site.GetService(typeof(ISelectionService));
				CheckNull(_selectionService,
					"SelectionService");
				return _selectionService;
			}
		}
		//---------------------------------------------------------------------------
		public static IToolboxService ToolboxService{
			get{
				IToolboxService _toolboxService = (IToolboxService)BorlandIDE.
					GetService(typeof(IToolboxService));
				CheckNull(_toolboxService,
					"ToolboxService");
				return _toolboxService;
			}
		}
		//---------------------------------------------------------------------------
		public static IOTAEditor CurrentEditor{
			get{
				IOTAModule module = OTAUtils.CurrentModule;
				if ( (module == null) || (module.FileName.ToLower() == "default.htm") ){
					return null;
				} else {
					return module.CurrentEditor;
				}
			}
		}
		//---------------------------------------------------------------------------
		public static IOTAModule CurrentModule{
			get{
				IOTAModuleServices moduleServices = ModuleServices;
				return moduleServices.CurrentModule;
			}
		}
		//---------------------------------------------------------------------------
		public static string CurrentModuleFileName{
			get{
				IOTAModule module = OTAUtils.CurrentModule;

				if (module == null) {
					return null;
				} else {
					return module.FileName;
				}
			}
		}
		//---------------------------------------------------------------------------
		public static string CurrentEditorFileName{
			get{
				IOTAModule module = OTAUtils.CurrentModule;

				if (module == null){
					return null;
				}else{
					IOTAEditor editor = CurrentEditor;

					if (editor == null){
						return null;
					}

					if (editor.FileName == null || editor.FileName == ""){
						return module.FileName;
					}

					return editor.FileName;
				}
			}
		}
		//---------------------------------------------------------------------------
	}
}
