unit VisibleTabsDlg;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, CheckLst, PaletteFrm, ComCtrls, Buttons, ToolWin;

type
  TVisibleTabsDialog = class(TForm)
    Button1: TButton;
    Button2: TButton;
    Button3: TSpeedButton;
    Button4: TSpeedButton;
    Tabs: TListView;
    Label1: TLabel;
    btnEnable: TSpeedButton;
    btnDisable: TSpeedButton;
    procedure Button3Click(Sender: TObject);
    procedure Button4Click(Sender: TObject);
    procedure TabsAdvancedCustomDrawItem(Sender: TCustomListView;
      Item: TListItem; State: TCustomDrawState; Stage: TCustomDrawStage;
      var DefaultDraw: Boolean);
    procedure TabsChange(Sender: TObject; Item: TListItem;
      Change: TItemChange);
    procedure TabsSelectItem(Sender: TObject; Item: TListItem;
      Selected: Boolean);
    procedure btnEnableClick(Sender: TObject);
    procedure btnDisableClick(Sender: TObject);
  private
    FLoading: Boolean;
  public
    class function Run: Boolean;
  end;

var
  VisibleTabsDialog: TVisibleTabsDialog;

implementation

uses TypInfo;

{$R *.dfm}

{ TVisibleTabsDialog }

class function TVisibleTabsDialog.Run: Boolean;
var
  I: Integer;
  Item: TListItem;
begin
  with Create(nil) do
  try
    FLoading := True;
    try
      for I := 0 to FloatingPaletteForm.PaletteTabs.Count - 1 do
      begin
        Item := Tabs.Items.Add;
        Item.Caption := FloatingPaletteForm.PaletteTabs[I];
        Item.Checked := FloatingPaletteForm.HiddenTabs.IndexOf(Item.Caption) < 0;
        Item.SubItems.Add(BooleanIdents[Item.Checked]);
        Item.SubItems.Add(BooleanIdents[FloatingPaletteForm.DisabledTabs.IndexOf(Item.Caption) < 0]);
      end;
    finally
      FLoading := False;
    end;

    Result := ShowModal = mrOK;
    if Result then
    begin
      FloatingPaletteForm.HiddenTabs.Clear;
      FloatingPaletteForm.DisabledTabs.Clear;
      for I := 0 to Tabs.Items.Count - 1 do
      begin
        Item := Tabs.Items[I];
        if not Item.Checked then
          FloatingPaletteForm.HiddenTabs.Add(Item.Caption);
        if Item.SubItems[1] = BooleanIdents[False] then
          FloatingPaletteForm.DisabledTabs.Add(Item.Caption);
      end;
      FloatingPaletteForm.SaveStateNecessary := True;
      FloatingPaletteForm.RefreshUI;
    end;
  finally
    Free;
  end;
end;

procedure TVisibleTabsDialog.Button3Click(Sender: TObject);
var
  I: Integer;
begin
  for I := 0 to Tabs.Items.Count - 1 do
    Tabs.Items[I].Checked := True;
end;

procedure TVisibleTabsDialog.Button4Click(Sender: TObject);
var
  I: Integer;
begin
  for I := 0 to Tabs.Items.Count - 1 do
    Tabs.Items[I].Checked := False;
end;

procedure TVisibleTabsDialog.TabsAdvancedCustomDrawItem(
  Sender: TCustomListView; Item: TListItem; State: TCustomDrawState;
  Stage: TCustomDrawStage; var DefaultDraw: Boolean);
begin
  if Stage = cdPrePaint then
    if Item.SubItems[1] = BooleanIdents[False] then
    begin
      Tabs.Canvas.Font.Style := Tabs.Canvas.Font.Style + [fsStrikeOut];
      Tabs.Canvas.Font.Color := clGrayText;
    end;
end;

procedure TVisibleTabsDialog.TabsChange(Sender: TObject; Item: TListItem;
  Change: TItemChange);
begin
  if not FLoading and (Change = ctState) then
    Item.SubItems[0] := BooleanIdents[Item.Checked]
end;

procedure TVisibleTabsDialog.TabsSelectItem(Sender: TObject;
  Item: TListItem; Selected: Boolean);
begin
  btnEnable.Enabled := Tabs.SelCount > 0;
  btnDisable.Enabled := Tabs.SelCount > 0;
end;

procedure TVisibleTabsDialog.btnEnableClick(Sender: TObject);
var
  I: Integer;
  R: TRect;
begin
  for I := 0 to Tabs.Items.Count - 1 do
    if Tabs.Items[I].Selected then
    begin
      Tabs.Items[I].SubItems[1] := BooleanIdents[True];
      R := Tabs.Items[I].DisplayRect(drBounds);
      Windows.InvalidateRect(Tabs.Handle, @R, True)
    end;
end;

procedure TVisibleTabsDialog.btnDisableClick(Sender: TObject);
var
  I: Integer;
  R: TRect;
begin
  for I := 0 to Tabs.Items.Count - 1 do
    if Tabs.Items[I].Selected then
    begin
      Tabs.Items[I].SubItems[1] := BooleanIdents[False];
      R := Tabs.Items[I].DisplayRect(drBounds);
      Windows.InvalidateRect(Tabs.Handle, @R, True)
    end;
end;

end.

